//
//  CalculatorViewController.m
//  Calculator
//
//  Created by Li Li on 2/3/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CalculatorViewController.h"
#import "CalculatorBrain.h"
#import "CalculatorBrainExt.h"
#import "StackExt.h"
#import <objc/runtime.h>
#include <sys/types.h>
#include <unistd.h>
#import <Social/Social.h>
#import <Accounts/Accounts.h>
#import "Note.h"

@interface CalculatorViewController() //class extension: like anonymous categories
@property (nonatomic) BOOL inMiddleEnterNum;   
//@property (nonatomic, strong) CalculatorBrain *brain;
@end

@implementation CalculatorViewController
@synthesize inMiddleEnterNum = _inMiddleEnterNum;
@synthesize display = _display;
@synthesize brain = _brain;
@synthesize calNote = _calNote;
@synthesize noteView = _noteView;

- (CalculatorBrain *) brain{
    if (_brain==nil) //lazy instantiation
        _brain = [[CalculatorBrain alloc] init];
       return _brain;    
}

- (iCloudNote *) calNote{
    if (_calNote==nil) //lazy instantiation
        _calNote = [iCloudNote alloc];
    return _calNote;
}

- (void)dataReloaded:(NSNotification *)notification {
    
    self.calNote.doc.noteContent = notification.object;
    self.noteView.text = self.calNote.doc.noteContent;
    
}

- (void)viewDidLoad{
    [super viewDidLoad];
    [self.calNote loadDocument]; //retrieve document from iCloud
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(dataReloaded:)
                                                 name:@"noteModified" object:nil];
    
    NSLog(@"Registering with the model for changes of property: arithExp, testKVO...");
    //Add observer
    [self.brain addObserver:self forKeyPath:@"arithExp" options:(NSKeyValueObservingOptionNew) context:@"myContext"]; //need to call willChangeValueForKey, didChangeValueForKey
    
    [self.brain addObserver:self forKeyPath:@"testKVO" options:(NSKeyValueObservingOptionNew) context:@"myContext"];  // Do NOT need to call willChangeValueForKey, didChangeValueForKey
}


- (void)textViewDidChange:(UITextView *)textView {
    
    self.calNote.doc.noteContent = textView.text;
    [self.calNote.doc updateChangeCount:UIDocumentChangeDone];
    
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    NSLog(@"noteContent: %@", self.calNote.doc.noteContent);
    self.noteView.text = self.calNote.doc.noteContent;
}

- (IBAction)handleTapFrom:(UITapGestureRecognizer *)sender {
    [self.noteView endEditing:YES];
}


- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object
                        change:(NSDictionary *)change
                       context:(void *)context{ //test KVO
    NSLog(@"really changed***");
    if (true || [keyPath isEqualToString:@"arithExp"] ) // process here
        NSLog(@"%@ changed: %@", keyPath, [change objectForKey:NSKeyValueChangeNewKey]);
    
    [self.brain removeObserver:self forKeyPath:keyPath];
}

- (IBAction)digitPressed:(UIButton*)sender {
      NSString *digit = [sender currentTitle];
    if(self.inMiddleEnterNum) {
        self.display.text = [self.display.text stringByAppendingString:digit];
        
        [self.brain appendOp:digit]; //calling updated model
    } else {
        self.display.text = digit;
        self.inMiddleEnterNum = YES;
        
        [self.brain appendOp:digit];
        NSString *associatedObject =self.brain.arithExp;
        NSLog(@"associatedObject: %@", associatedObject);
    }
}

- (IBAction)ClearDisplay:(UIButton*)sender {
    self.display.text = @"0";    
    [self.brain emptyStack];
    [self.brain setArithExp:nil];
}


- (IBAction)EnterPressed {
    [self.brain push:[self.display.text doubleValue]];
    
    self.inMiddleEnterNum = NO; 
    [self.brain appendOp:@" "];
    
}

- (IBAction)OperatorPressed:(UIButton *)sender {
    if(self.inMiddleEnterNum) [self EnterPressed];
    
    double result = [self.brain performOperation:[sender currentTitle]];
    NSString *resultString = [NSString stringWithFormat:@"%g",result];
 
    self.display.text = resultString;
    [self.brain appendOp:[sender currentTitle]];
}

- (IBAction)DisplayExpressionPressed:(UIButton*)sender {
    [self.brain appendOp:[NSString stringWithFormat:@"=%@ ", self.display.text]];
    self.display.text = self.brain.arithExp;
    self.calNote.doc.noteContent=[self.calNote.doc.noteContent stringByAppendingString:self.display.text];
    self.noteView.text = self.calNote.doc.noteContent;
   // [self SLRequestTest]; //test social API
}

- (void) SLRequestTest { //test the SLRequest API, more low level, but can do GET
    ACAccountStore *accountStore = [[ACAccountStore alloc] init];

    ACAccountType *facebookAccountType = [accountStore
                                      accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierFacebook];

    // Specify App ID and permissions
    NSDictionary *options = @{
    ACFacebookAppIdKey: @"283287448439342",
    //ACFacebookPermissionsKey: @[@"user_location", @"email"], //use this the first time!
    ACFacebookPermissionsKey: @[@"publish_stream", @"publish_actions"], //The Facebook server could not fulfill this access request: The app must ask for a basic read permission at install time.
    ACFacebookAudienceKey: ACFacebookAudienceOnlyMe //ACFacebookAudienceFriends
    };
    //[[ACAccountCredential alloc] initWithOAuthToken:kOAuth_Token tokenSecret:kOAuth_Token_Secret];
    __block ACAccount *facebookAccount;
    [accountStore requestAccessToAccountsWithType:facebookAccountType
                                          options:options completion:^(BOOL granted, NSError *e) {
                                          
                    if (granted) {
                        NSLog(@"account access succeeded.....");
                        self.display.text = @"Facebook account access succeeded";
                        NSArray *accounts = [accountStore accountsWithAccountType:facebookAccountType];
                        facebookAccount = [accounts lastObject];
                    }
                    else{
                        // Handle Failure
                        NSLog(@"account access failure.....");
                        self.display.text = @"Facebook account access failed";
                        NSLog(@"%@",e.domain);
                        for(NSString *key in e.userInfo) {
                            NSLog(@"userinfo, key=%@ value=%@",key, [e.userInfo objectForKey:key]);
                        }
                                              //NSDictionary *_userInfo;
                    }
    }];

    NSDictionary *parameters = @{@"message": @"My first iOS 6 Facebook posting "};

    NSURL *feedURL = [NSURL URLWithString:@"http://www.facebook.com/erran"];

    SLRequest *feedRequest = [SLRequest
                          requestForServiceType:SLServiceTypeFacebook
                          requestMethod:SLRequestMethodGET
                          // requestMethod:SLRequestMethodPOST
                          URL:feedURL
                          parameters:parameters];

    feedRequest.account = facebookAccount;

    [feedRequest performRequestWithHandler:^(NSData *responseData,
                                         NSHTTPURLResponse *urlResponse, NSError *error)
     {
     // Handle response
     // NSArray *response = [NSJSONSerialization JSONObjectWithData:responseData options:NSJSONReadingMutableLeaves error:&error];
         NSString *response = [[NSString alloc] initWithData:responseData encoding:NSUTF8StringEncoding];
         NSLog(@"feedRequest response, status code: %d, data:%@", urlResponse.statusCode, response);
     }];
}

- (IBAction)postToSocialNetwork:(UIButton *)sender {
    NSString *socialNetwork;
    
    if ([[sender currentTitle] isEqualToString:@"Facebook"])
        socialNetwork = SLServiceTypeFacebook;
    else if ([[sender currentTitle] isEqualToString:@"Twitter"])
        socialNetwork = SLServiceTypeTwitter;
    
    SLComposeViewController *socialController = [SLComposeViewController composeViewControllerForServiceType:socialNetwork];
    
    if([SLComposeViewController isAvailableForServiceType:socialNetwork]){
        SLComposeViewControllerCompletionHandler __block completionHandler=^(SLComposeViewControllerResult result){
            [socialController dismissViewControllerAnimated:YES completion:nil];
            
            switch(result){
                case SLComposeViewControllerResultCancelled:
                default:
                    NSLog(@"Cancelled.....");
                    break;
                case SLComposeViewControllerResultDone:
                    NSLog(@"Posted....");
                    break;
            }
        };
        
        [socialController addImage:[UIImage imageNamed:@"CollatzFractal.png"]];
        [socialController setInitialText:@"Solve the 3x+1 math puzzle."];
        [socialController addURL:[NSURL URLWithString:@"http://en.wikipedia.org/wiki/Collatz_conjecture"]];
        [socialController setCompletionHandler:completionHandler];
        //[self presentViewController:fbController animated:YES completion:nil];
        [self presentModalViewController:socialController animated:YES];
    }
}


- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (void)viewDidUnload {
    [self setNoteView:nil];
    [self setTapRecognizer:nil];
    [super viewDidUnload];
}
@end
